#
# Copyright (c) 2023, MediaTek Inc. All rights reserved.
#
# SPDX-License-Identifier: BSD-3-Clause
#

include lib/libfdt/libfdt.mk
include lib/xz/xz.mk

# BL2 image options
BL2_IS_AARCH64		:=	1
BL2_SIG_TYPE		:=	sha256+rsa-pss
BL2_IMG_HDR_SOC		:=	mt7986

# Whether to enble extra feature
I2C_SUPPORT		:= 	0
ENABLE_JTAG		?=	0

DTS_NAME		:=	mt7986
FDT_SOURCES		+=	fdts/$(DTS_NAME).dts
BL2_CPPFLAGS		+=	-DDTB_PATH=\"$(BUILD_PLAT)/fdts/$(DTS_NAME).dtb\"

BL2_CPPFLAGS		+=	-I$(MTK_PLAT)/common/drivers/spi		\
				-I$(MTK_PLAT_SOC)/drivers/gpio			\
				-I$(MTK_PLAT_SOC)/drivers/spi

BL2_SOURCES		+=	common/desc_image_load.c			\
				common/image_decompress.c			\
				drivers/delay_timer/delay_timer.c		\
				drivers/gpio/gpio.c				\
				drivers/io/io_storage.c				\
				drivers/io/io_block.c				\
				drivers/io/io_fip.c				\
				lib/cpus/aarch64/cortex_a53.S			\
				$(XZ_SOURCES)					\
				$(MTK_PLAT)/common/drivers/uart/aarch64/hsuart.S\
				$(MTK_PLAT)/common/mtk_plat_common.c		\
				$(MTK_PLAT)/common/drivers/gpt/mt_gpt.c		\
				$(MTK_PLAT)/common/drivers/spi/mtk_spi.c	\
				$(MTK_PLAT)/common/drivers/efuse/mtk_efuse.c	\
				$(MTK_PLAT_SOC)/aarch64/plat_helpers.S		\
				$(MTK_PLAT_SOC)/aarch64/platform_common.c	\
				$(MTK_PLAT_SOC)/bl2/dtb.S			\
				$(MTK_PLAT_SOC)/bl2/bl2_plat_init.c		\
				$(MTK_PLAT_SOC)/drivers/timer/timer.c		\
				$(MTK_PLAT_SOC)/drivers/gpio/mt7986_gpio.c	\
				$(MTK_PLAT_SOC)/drivers/spi/boot_spi.c		\
				$(MTK_PLAT_SOC)/drivers/pll/pll.c		\
				$(MTK_PLAT_SOC)/drivers/wdt/mtk_wdt.c

# Include dram driver files
include $(MTK_PLAT_SOC)/drivers/dram/dram.mk

ifeq ($(I2C_SUPPORT), 1)
include $(MTK_PLAT)/common/drivers/i2c/i2c.mk
endif

# Trusted board boot
include $(MTK_PLAT)/apsoc_common/bl2/tbbr.mk

# Anti-rollback
include $(MTK_PLAT)/apsoc_common/bl2/ar.mk

ifeq ($(BL2_COMPRESS),1)
BL2_CPPFLAGS		+=	-DUSING_BL2PL
endif # END OF BL2_COMPRESS

ifeq ($(ENABLE_JTAG), 1)
BL2_CPPFLAGS		+=	-DENABLE_JTAG
endif

BL2_BASE		:=	0x201000
BL2_CPPFLAGS		+=	-DBL2_BASE=$(BL2_BASE)

#
# Boot device related build macros
#
include $(MTK_PLAT)/apsoc_common/bl2/bl2_image.mk

ifndef BOOT_DEVICE
$(echo You must specify the boot device by provide BOOT_DEVICE= to \
	make parameter. Avaliable values: nor emmc sdmmc snand)
else # NO BOOT_DEVICE

ifeq ($(BOOT_DEVICE),ram)
$(eval $(call BL2_BOOT_RAM))
endif # END OF BOOT_DEVICE = ram

ifeq ($(BOOT_DEVICE),nor)
$(eval $(call BL2_BOOT_NOR))
BL2_SOURCES		+=	$(MTK_PLAT_SOC)/bl2/bl2_dev_qspi.c		\
				$(MTK_PLAT_SOC)/bl2/bl2_dev_spi_nor.c
endif # END OF BOOTDEVICE = nor

ifeq ($(BOOT_DEVICE),emmc)
$(eval $(call BL2_BOOT_EMMC))
BL2_SOURCES		+=	$(MTK_PLAT_SOC)/bl2/bl2_dev_mmc.c
BL2_CPPFLAGS		+=	-DMSDC_INDEX=0
endif # END OF BOOTDEVICE = emmc

ifeq ($(BOOT_DEVICE),sdmmc)
$(eval $(call BL2_BOOT_SD))
BL2_SOURCES		+=	$(MTK_PLAT_SOC)/bl2/bl2_dev_mmc.c
CPPFLAGS		+=	-DMSDC_INDEX=1
endif # END OF BOOTDEVICE = sdmmc

ifeq ($(BOOT_DEVICE),snand)
$(eval $(call BL2_BOOT_SNFI,0,0))
BL2_SOURCES		+=	$(MTK_PLAT_SOC)/bl2/bl2_dev_snfi.c
NAND_TYPE		?=	hsm:2k+64
$(eval $(call BL2_BOOT_NAND_TYPE_CHECK,$(NAND_TYPE),hsm:2k+64 hsm:2k+128 hsm:4k+256))
endif # END OF BOOTDEVICE = snand (snfi)

ifeq ($(BOOT_DEVICE),spim-nand)
$(eval $(call BL2_BOOT_SPI_NAND,0,0))
BL2_SOURCES		+=	$(MTK_PLAT_SOC)/bl2/bl2_dev_qspi.c		\
				$(MTK_PLAT_SOC)/bl2/bl2_dev_spi_nand.c
NAND_TYPE		?=	spim:2k+64
$(eval $(call BL2_BOOT_NAND_TYPE_CHECK,$(NAND_TYPE),spim:2k+64 spim:2k+128 spim:4k+256))
endif # END OF BOOTDEVICE = spim-nand

ifeq ($(BROM_HEADER_TYPE),)
$(error BOOT_DEVICE has invalid value. Please re-check.)
endif

endif # END OF BOOT_DEVICE
